import os
import sys
from jinja2 import Environment, PackageLoader
from subprocess import Popen
env = Environment(loader=PackageLoader('kindling', 'templates'))

FILES = (
    ('.gitignore', False),
    ('config.rb', False),
    ('requirements.txt', False),
    ('fabfile.py', False),
    ('settings/__init__.py', True),
    ('settings/base.py', True),
    ('settings/development.py', True),
    ('settings/local.py', True),
    ('settings/production.py', True),
    ('settings/celery_development.py', True),
    ('settings/celery_local.py', True),
    ('settings/celery_production.py', True),
    ('static/javascripts/default.build.js', True),
    ('static/javascripts/require.js', True),
    ('static/javascripts/jquery.js', True),
    ('static/javascripts/jquery.cookie.js', True),
    ('static/javascripts/main.build.js', True),
    ('static/javascripts/main.js', True),
)

def generate(app, path, in_app=False):
    print(u'Generating {}'.format(path))
    template = env.get_template(path)
    path = os.path.split(path)
    data = template.render(app=app)
    path = [app] + list(path)
    if in_app:
        path.insert(0, app)
    directory = os.path.join(*path[:-1])
    if not os.path.exists(directory):
        print("Made dir {}".format(directory))
        os.makedirs(directory)
    with open(os.path.join(*path), 'w') as f:
        f.write(data)

def new(app):
    os.unsetenv('DJANGO_SETTINGS_MODULE')
    if not app:
        print('App must be set.')
        sys.exit()
    
    if os.path.exists(app):
        print('App already exists.')
        sys.exit()
    
    p = Popen(['django-admin.py', 'startproject', app])
    p.wait()

    p = Popen(['git', 'init', app])
    p.wait()
    
    os.remove(os.path.join(app, app, 'settings.py'))
    for args in FILES:
        generate(app, *args)

    virtual_env = os.environ.get('VIRTUAL_ENV', None)
    if virtual_env:
        with open(os.path.join(virtual_env, 'bin', 'postactivate'), 'w') as f:
            f.write('#!/bin/zsh\nexport DJANGO_SETTINGS_MODULE={app}.settings.local'.format(app=app))


def main():
    new(sys.argv[1])


if __name__ == '__main__':
    main()



from base import *
DEBUG = True
INSTALLED_APPS += (
    'django_nose',
)
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
        'JOHNNY_CACHE': True,
    }
}

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'tests.db',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}
EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'
STATICFILES_STORAGE = 'django.contrib.staticfiles.storage.StaticFilesStorage'
NOSE_ARGS = ['--verbosity', '1']
TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
CELERY_ALWAYS_EAGER = True



# coding: utf-8
import os
from unipath import Path
import djcelery
from django.utils.translation import (
    ugettext_lazy as _,
)
import django.conf.global_settings as DEFAULT_SETTINGS

BROKER_URL = 'redis://localhost:6379/0'
djcelery.setup_loader()

APP = '{{ app }}'
DEBUG = False
TEMPLATE_DEBUG = DEBUG

PROJECT_ROOT = Path(__file__).ancestor(3)

ADMINS = (
)

LOGIN_URL = '/login/'
LOGOUT_URL = '/logout/'
LOGIN_REDIRECT_URL = '/'

MANAGERS = ADMINS

AUTH_USER_MODEL = DEFAULT_SETTINGS.AUTH_USER_MODEL

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': APP,
        'USER': APP,
        'PASSWORD': os.environ.get('DJANGO_DB_PASSWORD'),
        'HOST': '',
        'PORT': '',
        'OPTIONS': {
            'autocommit': True,
        },
    }
}

CACHES = {
    'default' : dict(
        BACKEND='johnny.backends.memcached.MemcachedCache',
        LOCATION=['127.0.0.1:11211'],
        JOHNNY_CACHE=True,
        KEY_PREFIX=APP,
    )
}

THUMBNAIL_QUALITY = 100
THUMBNAIL_COLORSPACE = None
THUMBNAIL_FORMAT = 'PNG'

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'Europe/London'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-gb'
LOCALE_PATHS = (
    PROJECT_ROOT.child('locale'),
)


SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"
MEDIA_ROOT = PROJECT_ROOT.child('media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = PROJECT_ROOT.child('static')

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = os.environ.get('DJANGO_SECRET_KEY', 'test')

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    # 'django.template.loaders.eggs.Loader',
    'apptemplates.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.transaction.TransactionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'breadcrumbs.middleware.BreadcrumbsMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
)
TEMPLATE_CONTEXT_PROCESSORS = DEFAULT_SETTINGS.TEMPLATE_CONTEXT_PROCESSORS + (
    "django.core.context_processors.request",
    #'{app}.context_processors.{app}'.format(app=APP),
)

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

STATICFILES_STORAGE = 'require.storage.OptimizedCachedStaticFilesStorage'

# require.js settings
REQUIRE_BASE_URL = "javascripts"
REQUIRE_BUILD_PROFILE = 'default.build.js'
REQUIRE_JS = "require.js"
REQUIRE_DEBUG = DEBUG
REQUIRE_ENVIRONMENT = 'node'
REQUIRE_STANDALONE_MODULES = {
    "main": {
        # Where to output the built module, relative to REQUIRE_BASE_URL.
        "out": "main-built.js",

        # Optional: A build profile used to build this standalone module.
        "build_profile": "main.build.js",
    }
}

BREADCRUMBS_AUTO_HOME = True
BREADCRUMBS_HOME_TITLE = _('Dashboard')


ROOT_URLCONF = '{app}.urls'.format(app=APP)

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = '{app}.wsgi.application'.format(app=APP)

INSTALLED_APPS = (
    APP,

    'breadcrumbs',
    'djcelery',
    'crispy_forms',
    'require',
    'sorl.thumbnail',
    'south',

    'django_extensions',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Uncomment the next line to enable the admin:
    'django.contrib.admin',
    # Uncomment the next line to enable admin documentation:
    # 'django.contrib.admindocs',
)


# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

SESSION_ENGINE = "django.contrib.sessions.backends.cached_db"

CELERYBEAT_SCHEDULER = 'djcelery.schedulers.DatabaseScheduler'



from .development import *


DATABASES['default']['NAME'] = '{{ app }}'
DATABASES['default']['USER'] = '{{ app }}'
DATABASES['default']['PASSWORD'] = '{{ app }}'
DATABASES['default']['ENGINE'] = 'django.db.backends.postgresql_psycopg2'
DATABASES['default']['OPTIONS'] = {
    'autocommit': True,
}
STATICFILES_STORAGE = 'django.contrib.staticfiles.storage.StaticFilesStorage'

SECRET_KEY = 'local'

INSTALLED_APPS += ('debug_toolbar',)
MIDDLEWARE_CLASSES += ('debug_toolbar.middleware.DebugToolbarMiddleware',)
INTERNAL_IPS = ('127.0.0.1',)



from .base import *

DATABASES['default']['NAME'] = '{{ app }}_test'
DATABASES['default']['USER'] = '{{ app }}_test'
DATABASES['default']['ENGINE'] = 'django.db.backends.mysql'
DATABASES['default']['OPTIONS'] = {}

DEBUG = True
TEMPLATE_DEBUG = DEBUG
REQUIRE_DEBUG = DEBUG

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
        'JOHNNY_CACHE': True,
    }
}
EMAIL_BACKEND = 'debugmail.backends.DebugEmailBackend'

HTTP_SECURE = False
SENDFILE_BACKEND = 'sendfile.backends.development'



from .settings import *

DATABASES['default']['NAME'] = '{{ app }}_live'
DATABASES['default']['USER'] = '{{ app }}_live'
DATABASES['default']['ENGINE'] = 'django.db.backends.mysql'
DATABASES['default']['OPTIONS'] = {}

# django-secure settings
SECURE_SSL_REDIRECT = True
SESSION_COOKIE_SECURE = True
SECURE_HSTS_SECONDS = 31536000
SECURE_HSTS_INCLUDE_SUBDOMAINS = True
SECURE_FRAME_DENY = True
SECURE_CONTENT_TYPE_NOSNIFF = True
SECURE_BROWSER_XSS_FILTER = True


INSTALLED_APPS = INSTALLED_APPS + (
    'djangosecure',
)


from fabric.api import *
env.ignore_virtualenv_override = True
from links_devops import (
    celery,
    debug,
    init,
    initialise as _initialise,
    manage,
    restart,
    shell,
    upgrade,
    virtualenv,
)

env.project = '{{ project }}'
env.repo = '{{ app }}'
env.app = env.repo
env.application = 'django'
env.debug_port = 8000
env.celery = False
env.memcached = True


def test(app=None):
    app = app or '{{ app }} customers instruments reports'
    local('REUSE_DB=1 ./manage.py test {} --settings={{ app }}.settings.test --with-color --with-fixture-bundling'.format(app))


def initialise(instance, *args, **kwargs):
    if instance == 'test':
        env.domains = ['{{ app }}.linkscreative.co.uk']
    elif instance == 'live':
        env.domains = ['{{ app }}-live.linkscreative.co.uk']
        
    _initialise(instance, *args, **kwargs)
    with virtualenv():
        run('mkdir -p media/uploads')
        if instance == 'test':
            manage('loaddata testdata')